/*
 *  Licensed Materials - Property of IBM Corp.
 *  5648-C60 (c) Copyright IBM Corp. 1999
 *
 *  Point to Point Sample03
 *
 * Demonstrates:
 *   the use of a QueueRequester.
 *
 * This program creates two threads, one uses a QueueRequester to request
 * work from a server, the other acts as the server and creates a response
 * for each request.
 * The program is configured to use the default queue manager and
 * the SYSTEM.DEFAULT.LOCAL.QUEUE.
 *
 *  Usage:
 *
 * java PTPSample03
 *
 */

// The main JMS interfaces
import javax.jms.*;
// vendor specific connection factory
import com.ibm.mq.jms.MQQueueConnectionFactory;

public class PTPSample03 extends Thread
{
  // the number of messages to be sent between requester and responder
  static final int NMSGS=3;

  // enumerate the modes of operation
  static final int REQUESTER=0;
  static final int RESPONDER=1;

  // the queue manager to use for the test
  static final String QMGR = "";

  // the queue to use for the test
  static final String testQueue = "SYSTEM.DEFAULT.LOCAL.QUEUE";

  // which mode this thread should run in.
  int mode;


  // constructor
  public PTPSample03(int mode) {
    this.mode = mode;
  }

  /** main() entry point.
      Creates two instances of this class, one to act as the
      requester (client), the other to act as a responder (server).
  */
  static public void main(String[] args) {

    // create two threads
    PTPSample03 requester = new PTPSample03(REQUESTER);
    PTPSample03 responder = new PTPSample03(RESPONDER);

    // start each running
    requester.start();
    responder.start();

    // since the threads are not marked as 'daemon', the JVM won't exit
    // until both threads have finished.
  }

  /** The entry point for each thread.
      The calls to Thread.start() result in this method being called.
  */
  public void run()
  {
    // Check which mode the thread is running in and call the
    // appropriate method.
    switch (mode) {
    case REQUESTER:
      makeRequests();
      break;

    case RESPONDER:
      respond();
      break;

    default:
      System.err.println("unknown value for mode: "+mode);
    }
  }

  /** Use a QueueRequester.
      Act as a client to request work from a server.
   */
  void makeRequests()
  {
    try {
      System.out.println("Client Started");

      // create a new connection factory
      MQQueueConnectionFactory factory = new MQQueueConnectionFactory();
      factory.setQueueManager(QMGR);

      // create a connection
      QueueConnection connection = factory.createQueueConnection();
      // start the connection to enable receives
      connection.start();

      // create a session
      QueueSession session =
	connection.createQueueSession(false,
				      Session.AUTO_ACKNOWLEDGE);

      // create a destination object
      Queue queue = session.createQueue(testQueue);

      // create a requestor
      QueueRequestor qReq = new QueueRequestor(session, queue);

      // use the requestor to interact with the server
      TextMessage outMessage = session.createTextMessage();
      for(int i=0; i<NMSGS; i++) {
	outMessage.setText("request message "+i);

        // send the request and get the reply
	Message replyMessage = qReq.request(outMessage);

        // check and display the reply
	if (replyMessage == null) {
	  System.out.println("QueueRequestor returned a null");
	} else if (replyMessage instanceof TextMessage) {
	  String replyString = ((TextMessage)replyMessage).getText();
	  System.out.println("reply was '"+replyString+"'");
	} else {
	  System.out.println("replyMessage not a TextMessage:");
	  System.out.println(replyMessage);
	}
      }

      // close down
      session.close();
      connection.close();

    } catch (JMSException je) {
      System.err.println("Client caught "+je);
      Exception e = je.getLinkedException();
      if (e != null) System.err.println("linked exception: "+e);
    }

    System.out.println("Client Finished");
  }


  /** reply to messages sent by the requester.
   */
  void respond()
  {
    try {
      System.out.println("Server Started");

      // create the connection factory
      MQQueueConnectionFactory factory = new MQQueueConnectionFactory();
      factory.setQueueManager(QMGR);

      // create a connection
      QueueConnection connection = factory.createQueueConnection();
      // start the connection to enable receives
      connection.start();

      // create a session
      QueueSession session =
	connection.createQueueSession(false,
				      Session.AUTO_ACKNOWLEDGE);

      // create a destination object
      Queue queue = session.createQueue(testQueue);

      // create a receiver
      QueueReceiver receiver = session.createReceiver(queue);
      
      // create an unidentified sender for returning the replies.
      QueueSender sender = session.createSender(null);
      sender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

      // get each message and send a response
      for(int i=0; i<NMSGS; i++) {

        // blocking receive
	Message requestMessage = receiver.receive();

	if (requestMessage instanceof TextMessage) {
	  // extract the text from the request
	  String requestString = ((TextMessage)requestMessage).getText();
	  // create a reply
	  TextMessage replyMessage = session.createTextMessage();
	  replyMessage.setText("this is a reply to '"+requestString+"'");

	  // get the replyTo destination from the request
	  Queue replyTo = (Queue)requestMessage.getJMSReplyTo();

	  // send the reply message to the replyTo destination
	  sender.send(replyTo, replyMessage);

	} else {
	  System.out.println("requestMessage not a TextMessage:");
	  System.out.println(requestMessage);
	}
      }

      // close down
      session.close();
      connection.close();

    } catch (JMSException je) {
      System.err.println("Server caught "+je);
      Exception e = je.getLinkedException();
      if (e != null) System.err.println("linked exception: "+e);
    }

    System.out.println("Server Finished");
  }


}









